package com.icomsys.main_vm.db.mybatis.mapper;

import org.apache.ibatis.annotations.Mapper;

import java.util.HashMap;

@Mapper
public interface NlpWisenutApiMapper {

    long selectProjectId(HashMap eMap);
    long selectProjectIdforOprMngCode(HashMap eMap);
    HashMap selectProjectName(HashMap eMap);
    long selectIntentId(HashMap eMap);
    HashMap selectIntentIdForDelete(HashMap eMap);
    long selectIntentSentenctId(HashMap eMap);
    HashMap selectIntentSentenctIdforDelete(HashMap eMap);
    long selectEntityId(HashMap eMap);
    HashMap selectEntityIdForDelete(HashMap eMap);
    long selectDictionaryId(HashMap eMap);
    HashMap selectDictionaryIdforDelete(HashMap eMap);

}
